@@ -31,8 +31,8 @@ namespace import stooop::*
 class a {}
 proc a::a {this p q} {
     puts "a::a $this"
-    set a::($this,m) $p
-    set a::($this,n) $q
+    set ($this,m) $p
+    set ($this,n) $q
 }
 proc a::~a {this} {
     puts "a::~a $this"
@@ -45,8 +45,8 @@ printArrays a::
 class A {
     proc A {this p q} {
         puts "A::A $this"
-        set A::($this,m) $p
-        set A::($this,n) $q
+        set ($this,m) $p
+        set ($this,n) $q
     }
     proc ~A {this} {
         puts "A::~A $this"
@@ -60,8 +60,8 @@ printArrays A::
 class c::d {}
 proc c::d::d {this p q} {
     puts "d::d $this"
-    set c::d::($this,m) $p
-    set c::d::($this,n) $q
+    set ($this,m) $p
+    set ($this,n) $q
 }
 proc c::d::~d {this} {
     puts "d::~d $this"
@@ -75,8 +75,8 @@ class C {
     class D {
         proc D {this p q} {
             puts "D::D $this"
-            set C::D::($this,m) $p
-            set C::D::($this,n) $q
+            set ($this,m) $p
+            set ($this,n) $q
         }
         proc ~D {this} {
             puts "D::~D $this"
