@@ -14,14 +14,14 @@ namespace import stooop::*
 
 class a {}
 proc a::a {this} {
-    set a::($this,x) 0
+    set ($this,x) 0
 }
 new [new a]
 printArrays a::
 
 class A {
     proc A {this} {
-        set A::($this,x) 0
+        set ($this,x) 0
     }
 }
 new [new A]
