@@ -14,20 +14,20 @@ namespace import stooop::*
 
 class a {}
 proc a::a {this} {
-    set a::($this,x) 0
+    set ($this,x) 0
 }
 proc a::a {this copy} {
-    set a::($this,x) [expr $a::($copy,x)+1]
+    set ($this,x) [expr $($copy,x)+1]
 }
 new [new a]
 printArrays a::
 
 class A {
     proc A {this} {
-        set A::($this,x) 0
+        set ($this,x) 0
     }
     proc A {this copy} {
-        set A::($this,x) [expr $A::($copy,x)+1]
+        set ($this,x) [expr $($copy,x)+1]
     }
 }
 new [new A]
