@@ -12,7 +12,7 @@ namespace import stooop::*
 catch {
     class a {}
     proc a::a {this} {
-        set a::($this,x) 0
+        set ($this,x) 0
     }
     proc a::a {destination source} {}
     new [new a]
@@ -22,7 +22,7 @@ puts $message
 catch {
     class A {
         proc A {this} {
-            set A::($this,x) 0
+            set ($this,x) 0
         }
         proc A {destination source} {}
     }
@@ -34,7 +34,7 @@ catch {
     class b {}
     class b::c {}
     proc b::c::c {this} {
-        set b::c::($this,x) 0
+        set ($this,x) 0
     }
     proc b::c::c {destination source} {}
     new [new b::c]
@@ -45,7 +45,7 @@ catch {
     class B {
         class C {
             proc C {this} {
-                set B::C::($this,x) 0
+                set ($this,x) 0
             }
             proc C {destination source} {}
         }
