@@ -13,11 +13,11 @@ class a {
     proc ~a {this} {}
 }
 proc a::p {this p} {
-    set a::($this,m) $p
+    set ($this,m) $p
 }
 class a {
     proc q {this} {
-        puts $a::($this,m)
+        puts $($this,m)
     }
 }
 set o [new a]
@@ -30,11 +30,11 @@ class b {
         proc ~c {this} {}
     }
     proc c::p {this p} {
-        set b::c::($this,m) $p
+        set ($this,m) $p
     }
     class c {
         proc q {this} {
-            puts $b::c::($this,m)
+            puts $($this,m)
         }
     }
     set o [new c]
