@@ -13,10 +13,10 @@ namespace import stooop::*
 ## B::C::(l) = 7 8 9 10
 
 class a {
-    set a::(l) {}
+    set (l) {}
 }
 proc a::a {this} {
-    lappend a::(l) $this
+    lappend (l) $this
 }
 proc a::~a {this} {}
 new a
@@ -26,7 +26,7 @@ printArrays a::
 class A {
     set A::(l) {}
     proc A {this} {
-        lappend A::(l) $this
+        lappend (l) $this
     }
     proc ~A {this} {}
 }
@@ -36,10 +36,10 @@ printArrays A::
 
 class b {}
 class b::c {
-    set b::c::(l) {}
+    set (l) {}
 }
 proc b::c::c {this} {
-    lappend b::c::(l) $this
+    lappend (l) $this
 }
 proc b::c::~c {this} {}
 new b::c
@@ -48,9 +48,9 @@ printArrays b::c::
 
 class B {
     class C {
-        set B::C::(l) {}
+        set (l) {}
         proc C {this} {
-            lappend B::C::(l) $this
+            lappend (l) $this
         }
         proc ~C {this} {}
     }
