@@ -20,10 +20,10 @@ class a {}
 proc a::a {this} {}
 class a::b {}
 proc a::b::b {this} {
-    set a::b::($this,x) 0
+    set ($this,x) 0
 }
 proc a::b::b {this copy} {
-    set a::b::($this,x) [expr $a::b::($copy,x)+1]
+    set ($this,x) [expr $($copy,x)+1]
 }
 new [new a::b]
 printArrays a::b::
@@ -32,10 +32,10 @@ class A {
     proc A {this} {}
     class B {
         proc B {this} {
-            set A::B::($this,x) 0
+            set ($this,x) 0
         }
         proc B {this copy} {
-            set A::B::($this,x) [expr $A::B::($copy,x)+1]
+            set ($this,x) [expr $($copy,x)+1]
         }
     }
     new [new B]
