@@ -20,7 +20,7 @@ class a {}
 proc a::a {this} {}
 class a::b {}
 proc a::b::b {this} {
-    set a::b::($this,x) 0
+    set ($this,x) 0
 }
 new [new a::b]
 printArrays a::b::
@@ -29,7 +29,7 @@ class A {
     proc A {this} {}
     class B {
         proc B {this} {
-            set A::B::($this,x) 0
+            set ($this,x) 0
         }
     }
     new [new B]
