@@ -1,5 +1,5 @@
 foreach name [array names env STOOOP*] {unset env($name)}                                ;# reset any existing environment variables
-set env(STOOOPCHECKPROCEDURES) {}
+set env(STOOOPCHECKPROCEDURES) 1
 source stooop.tcl
 namespace import stooop::*
 
