@@ -1,5 +1,5 @@
 foreach name [array names env STOOOP*] {unset env($name)}                                ;# reset any existing environment variables
-set env(STOOOPCHECKDATA) {}
+set env(STOOOPCHECKDATA) 1
 source stooop.tcl
 namespace import stooop::*
 
@@ -7,13 +7,13 @@ namespace import stooop::*
 
 class a {}
 proc a::a {this} {
-    set a::($this,n) 0
+    set ($this,n) 0
 }
 new [new a]
 
 class A {
     proc A {this} {
-        set A::($this,n) 0
+        set ($this,n) 0
     }
 }
 new [new A]
@@ -21,14 +21,14 @@ new [new A]
 class b {}
 class b::c {}
 proc b::c::c {this} {
-    set b::c::($this,n) 0
+    set ($this,n) 0
 }
 new [new b::c]
 
 class B {
     class C {
         proc C {this} {
-            set B::C::($this,n) 0
+            set ($this,n) 0
         }
     }
     new [new C]
