@@ -1,7 +1,13 @@
-if {[catch {package require stooop 3.0}]} {
-set rcsId {$Id: stooop.tcl,v 3.56 1998/02/04 21:30:14 jfontain Exp $}
+if {[catch {package require stooop 4.0}]} {
+set rcsId {$Id: stooop.tcl,v 4.5 2000/08/09 19:16:59 jfontain Exp $}
+
+package provide stooop 4.1.1
+
+if {[package vcompare $::tcl_version 8.3]<0} {
+    puts stderr "stooop version [package provide stooop] requires Tcl 8.3 or above"
+    exit 1
+}
 
-package provide stooop 3.3
 
 catch {rename proc _proc}
 
@@ -15,7 +21,7 @@ namespace eval ::stooop {
 
     set checkCode {}
     if {[info exists ::env(STOOOPCHECKALL)]} {
-        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {}}
+        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {} STOOOPCHECKOBJECTS {}}
     }
     if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
         append checkCode {::stooop::checkProcedure;}
@@ -56,9 +62,8 @@ namespace eval ::stooop {
 
         if {[scan $classOrId %u dummy]==0} {
             set constructor ${classOrId}::[namespace tail $classOrId]
-            set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-            set fullClass([set id [incr newId]]) $fullName
-            uplevel $constructor $id $args
+            uplevel $constructor [set id [incr newId]] $args
+            set fullClass($id) [namespace qualifiers [uplevel namespace which -command $constructor]]
         } else {
             if {[catch {set fullClass([set id [incr newId]]) $fullClass($classOrId)}]} {
                 error "invalid object identifier $classOrId"
@@ -79,9 +84,7 @@ namespace eval ::stooop {
 
     _proc deleteObject {fullClass id} {
         uplevel ${fullClass}::~[namespace tail $fullClass] $id
-        foreach name [array names ${fullClass}:: $id,*] {
-            unset ${fullClass}::($name)
-        }
+        array unset ${fullClass}:: $id,*
     }
 
     _proc classof {id} {
@@ -92,8 +95,8 @@ namespace eval ::stooop {
 
     _proc copy {fullClass from to} {
         set index [string length $from]
-        foreach name [array names ${fullClass}:: $from,*] {
-            set ${fullClass}::($to[string range $name $index end]) [set ${fullClass}::($name)]
+        foreach {name value} [array get ${fullClass}:: $from,*] {
+            set ${fullClass}::($to[string range $name $index end]) $value
         }
     }
 }
@@ -116,7 +119,7 @@ _proc ::stooop::parseProcedureName {namespace name fullClassVariable procedureVa
         return 1
     } else {
         if {![string match ::* $name]} {
-            if {[string compare $namespace ::]==0} {
+            if {[string equal $namespace ::]} {
                 set name ::$name
             } else {
                 set name ${namespace}::$name
@@ -140,20 +143,20 @@ _proc ::stooop::parseProcedureName {namespace name fullClassVariable procedureVa
 _proc ::stooop::virtual {keyword name arguments args} {
     variable pureVirtual
 
-    if {[string compare [uplevel namespace which -command $keyword] ::proc]!=0} {
+    if {![string equal [uplevel namespace which -command $keyword] ::proc]} {
         error "virtual operator works only on proc, not $keyword"
     }
     if {![parseProcedureName [uplevel namespace current] $name fullClass procedure message]} {
         error $message
     }
     set class [namespace tail $fullClass]
-    if {[string compare $class $procedure]==0} {
+    if {[string equal $class $procedure]} {
         error "cannot make class $fullClass constructor virtual"
     }
-    if {[string compare ~$class $procedure]==0} {
+    if {[string equal ~$class $procedure]} {
         error "cannot make class $fullClass destructor virtual"
     }
-    if {[string compare [lindex $arguments 0] this]!=0} {
+    if {![string equal [lindex $arguments 0] this]} {
         error "cannot make static procedure $procedure of class $fullClass virtual"
     }
     set pureVirtual [expr {[llength $args]==0}]
@@ -170,11 +173,11 @@ _proc proc {name arguments args} {
         error "missing body for ${fullClass}::$procedure"
     }
     set class [namespace tail $fullClass]
-    if {[string compare $class $procedure]==0} {
-        if {[string compare [lindex $arguments 0] this]!=0} {
+    if {[string equal $class $procedure]} {
+        if {![string equal [lindex $arguments 0] this]} {
             error "class $fullClass constructor first argument must be this"
         }
-        if {[string compare [lindex $arguments 1] copy]==0} {
+        if {[string equal [lindex $arguments 1] copy]} {
             if {[llength $arguments]!=2} {
                 error "class $fullClass copy constructor must have 2 arguments exactly"
             }
@@ -186,11 +189,11 @@ _proc proc {name arguments args} {
             eval ::stooop::constructorDeclaration $fullClass $class 0 \{$arguments\} $args
             ::stooop::generateDefaultCopyConstructor $fullClass
         }
-    } elseif {[string compare ~$class $procedure]==0} {
+    } elseif {[string equal ~$class $procedure]} {
         if {[llength $arguments]!=1} {
             error "class $fullClass destructor must have 1 argument exactly"
         }
-        if {[string compare [lindex $arguments 0] this]!=0} {
+        if {![string equal [lindex $arguments 0] this]} {
             error "class $fullClass destructor argument must be this"
         }
         if {[catch {info body ::${fullClass}::$class}]} {
@@ -214,7 +217,7 @@ _proc ::stooop::constructorDeclaration {fullClass class copy arguments args} {
     if {($number%2)==0} {
         error "bad class $fullClass constructor declaration, a base class, contructor arguments or body may be missing"
     }
-    if {[string compare [lindex $arguments end] args]==0} {
+    if {[string equal [lindex $arguments end] args]} {
         set variable($fullClass) {}
     }
     if {!$copy} {
@@ -237,7 +240,7 @@ _proc ::stooop::constructorDeclaration {fullClass class copy arguments args} {
             }
             lappend fullBases($fullClass) $fullBase
         }
-        regsub -all \n $baseArguments {} constructorArguments($fullBase)
+        regsub -all {\n} $baseArguments { } constructorArguments($fullBase)
     }
     set constructorBody \
 "::variable {}
@@ -279,7 +282,7 @@ $checkCode
     }
     if {$copy} {
         append constructorBody \
-"::catch {::set ${fullClass}::(\$this,_derived) \[::set ${fullClass}::(\$[::lindex $arguments 1],_derived)\]}
+"::catch {::set (\$this,_derived) \$(\$[::lindex $arguments 1],_derived)}
 "
     }
     append constructorBody [lindex $args end]
@@ -317,7 +320,7 @@ _proc ::stooop::memberProcedureDeclaration {fullClass class procedure arguments
             _proc ${fullClass}::$procedure $arguments \
 "::variable {}
 $checkCode
-::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel \$(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         } else {
             _proc ${fullClass}::_$procedure $arguments \
@@ -328,10 +331,10 @@ $body
             _proc ${fullClass}::$procedure $arguments \
 "::variable {}
 $checkCode
-if {!\[::catch {::info body \$${fullClass}::(\$this,_derived)::$procedure}\]} {
-::return \[::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
+if {!\[::catch {::info body \$(\$this,_derived)::$procedure}\]} {
+::return \[::uplevel \$(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
 }
-::eval ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         }
     } else {
@@ -385,23 +388,51 @@ if {[llength [array names ::env STOOOP*]]>0} {
                 set interface($fullClass) {}
             }
         }
+    }
 
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        _proc invokingProcedure {} {
+            if {[catch {set procedure [lindex [info level -2] 0]}]} {
+                return {main script}
+            } elseif {[string length $procedure]==0} {
+                return "namespace [uplevel 2 namespace current]"
+            } else {
+                return [uplevel 3 namespace which -command $procedure]
+            }
+        }
+    }
+
+    if {[info exists ::env(STOOOPCHECKPROCEDURES)]||[info exists ::env(STOOOPCHECKOBJECTS)]} {
         catch {rename ::stooop::new ::stooop::_new}
         _proc ::stooop::new {classOrId args} {
-            variable fullClass
-            variable interface
-
-            if {[scan $classOrId %u dummy]==0} {
-                set constructor ${classOrId}::[namespace tail $classOrId]
-                set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-            } else {
-                set fullName $fullClass($classOrId)
+            variable newId
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                variable fullClass
+                variable interface
             }
-            if {[info exists interface($fullName)]} {
-                error "class $fullName with pure virtual procedures should not be instanciated"
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                variable creator
+            }
+
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                if {[scan $classOrId %u dummy]==0} {
+                    set constructor ${classOrId}::[namespace tail $classOrId]
+                    catch {$constructor}
+                    set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
+                    set fullClass([expr {$newId+1}]) $fullName
+                } else {
+                    set fullName $fullClass($classOrId)
+                }
+                if {[info exists interface($fullName)]} {
+                    error "class $fullName with pure virtual procedures should not be instanciated"
+                }
+            }
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                set creator([expr {$newId+1}]) [invokingProcedure]
             }
             return [uplevel ::stooop::_new $classOrId $args]
         }
+
     }
 
     _proc ::stooop::ancestors {fullClass} {
@@ -433,12 +464,12 @@ if {[llength [array names ::env STOOOP*]]>0} {
         set procedure [lindex $list 0]
         set fullProcedure [uplevel 3 namespace which -command $procedure]
         set procedure [namespace tail $procedure]
-        if {[string compare $class $procedure]==0} {
+        if {[string equal $class $procedure]} {
             set procedure constructor
-        } elseif {[string compare ~$class $procedure]==0} {
+        } elseif {[string equal ~$class $procedure]} {
             set procedure destructor
         }
-        if {[string compare [lindex [info args $fullProcedure] 0] this]==0} {
+        if {[string equal [lindex [info args $fullProcedure] 0] this]} {
             set thisParameter [lindex $list 1]
         }
     }
@@ -448,11 +479,12 @@ if {[llength [array names ::env STOOOP*]]>0} {
 
         debugInformation class qualifiedClass procedure qualifiedProcedure this
         if {![info exists this]} return
+        if {[string equal $procedure constructor]} return
         if {![info exists fullClass($this)]} {
             error "$this is not a valid object identifier"
         }
         set fullName [string trimleft $fullClass($this) :]
-        if {[string compare $fullName $qualifiedClass]==0} return
+        if {[string equal $fullName $qualifiedClass]} return
         if {[lsearch -exact [ancestors ::$fullName] ::$qualifiedClass]<0} {
             error "class $qualifiedClass of $qualifiedProcedure procedure not an ancestor of object $this class $fullName"
         }
@@ -480,19 +512,19 @@ if {[llength [array names ::env STOOOP*]]>0} {
 
     _proc ::stooop::checkData {array name operation} {
         scan $name %u,%s identifier member
-        if {[info exists member]&&([string compare $member _derived]==0)} return
+        if {[info exists member]&&[string equal $member _derived]} return
 
         debugInformation class qualifiedClass procedure qualifiedProcedure this
         if {![info exists class]} return
         set array [uplevel [list namespace which -variable $array]]
         if {![info exists procedure]} {
-            if {[string compare $array ::${qualifiedClass}::]!=0} {
+            if {![string equal $array ::${qualifiedClass}::]} {
                 error "class access violation in class $qualifiedClass namespace"
             }
             return
         }
-        if {[string compare $qualifiedProcedure ::stooop::copy]==0} return
-        if {[string compare $array ::${qualifiedClass}::]!=0} {
+        if {[string equal $qualifiedProcedure ::stooop::copy]} return
+        if {![string equal $array ::${qualifiedClass}::]} {
             error "class access violation in procedure $qualifiedProcedure"
         }
         if {![info exists this]} return
@@ -507,9 +539,9 @@ if {[llength [array names ::env STOOOP*]]>0} {
         variable traceDataFormat
 
         scan $name %u,%s identifier member
-        if {[info exists member]&&([string compare $member _derived]==0)} return
+        if {[info exists member]&&[string equal $member _derived]} return
 
-        if {![catch {lindex [info level -1] 0} procedure]&&([string compare ::stooop::deleteObject $procedure]==0)} return
+        if {![catch {lindex [info level -1] 0} procedure]&&[string equal ::stooop::deleteObject $procedure]} return
         set class {}
         set qualifiedClass {}
         set procedure {}
@@ -534,38 +566,82 @@ if {[llength [array names ::env STOOOP*]]>0} {
         }
         array set string {r read w write u unset}
         regsub -all %o $text $string($operation) text
-        if {[string compare $operation u]==0} {
+        if {[string equal $operation u]} {
             regsub -all %v $text {} text
         } else {
             regsub -all %v $text [uplevel set ${array}($name)] text
         }
         puts $traceDataChannel $text
     }
+
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        _proc ::stooop::printObjects {{pattern *}} {
+            variable fullClass
+            variable creator
+
+            puts "stooop::printObjects invoked from [invokingProcedure]:"
+            foreach id [lsort -integer [array names fullClass]] {
+                if {[string match $pattern $fullClass($id)]} {
+                    puts "$fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+
+        _proc ::stooop::record {} {
+            variable fullClass
+            variable checkpointFullClass
+
+            puts "stooop::record invoked from [invokingProcedure]"
+            catch {unset checkpointFullClass}
+            array set checkpointFullClass [array get fullClass]
+        }
+
+        _proc ::stooop::report {{pattern *}} {
+            variable fullClass
+            variable checkpointFullClass
+            variable creator
+
+            puts "stooop::report invoked from [invokingProcedure]:"
+            set checkpointIds [lsort -integer [array names checkpointFullClass]]
+            set currentIds [lsort -integer [array names fullClass]]
+            foreach id $currentIds {
+                if {[string match $pattern $fullClass($id)]&&([lsearch -exact $checkpointIds $id]<0)} {
+                    puts "+ $fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+            foreach id $checkpointIds {
+                if {[string match $pattern $checkpointFullClass($id)]&&([lsearch -exact $currentIds $id]<0)} {
+                    puts "- $checkpointFullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+    }
+
 }
 }
 namespace import stooop::*
 
 class graphic {
     proc graphic {this canvas item} {
-        set graphic::($this,canvas) $canvas
-        set graphic::($this,item) $item
+        set ($this,canvas) $canvas
+        set ($this,item) $item
         $canvas bind $item <Button1-Motion> "graphic::moveTo $this %x %y"
         $canvas bind $item <ButtonRelease-1> "capture $this"
     }
 
     proc ~graphic {this} {
-        $graphic::($this,canvas) delete $graphic::($this,item)
+        $($this,canvas) delete $($this,item)
     }
 
     virtual proc add {this object}
 
     virtual proc moveBy {this x y} {
-        $graphic::($this,canvas) move $graphic::($this,item) $x $y
+        $($this,canvas) move $($this,item) $x $y
     }
 
     virtual proc moveTo {this x y} {
-        set coordinates [$graphic::($this,canvas) coords $graphic::($this,item)]
-        $graphic::($this,canvas) move $graphic::($this,item) [expr {$x-[lindex $coordinates 0]}] [expr {$y-[lindex $coordinates 1]}]
+        set coordinates [$($this,canvas) coords $($this,item)]
+        $($this,canvas) move $($this,item) [expr {$x-[lindex $coordinates 0]}] [expr {$y-[lindex $coordinates 1]}]
     }
 }
 
@@ -573,7 +649,7 @@ class picture {
     proc picture {this canvas} graphic {
         $canvas [$canvas create rectangle 0 0 100 80 -outline red -width 3]
     } {
-        set picture::($this,graphics) {}
+        set ($this,graphics) {}
     }
 
     proc ~picture {this} {}
@@ -587,8 +663,8 @@ class picture {
         set y [lindex $coordinates 1]
         set box [$graphic::($this,canvas) bbox $graphic::($this,item)]
         if {($x>=[lindex $box 0])&&($x<=[lindex $box 2])&&($y>=[lindex $box 1])&&($y<=[lindex $box 3])} {
-            if {[lsearch -exact $picture::($this,graphics) $object]<0} {
-                lappend picture::($this,graphics) $object
+            if {[lsearch -exact $($this,graphics) $object]<0} {
+                lappend ($this,graphics) $object
             }
             return 1
         }
@@ -596,7 +672,7 @@ class picture {
     }
 
     proc moveBy {this x y} {
-        foreach object $picture::($this,graphics) {
+        foreach object $($this,graphics) {
             graphic::moveBy $object $x $y
         }
         graphic::_moveBy $this $x $y
