@@ -1,6 +1,12 @@
-set rcsId {$Id: stooop.tcl,v 3.56 1998/02/04 21:30:14 jfontain Exp $}
+set rcsId {$Id: stooop.tcl,v 4.5 2000/08/09 19:16:59 jfontain Exp $}
+
+package provide stooop 4.1.1
+
+if {[package vcompare $::tcl_version 8.3]<0} {                     ;# check whether empty named arrays and array unset are supported
+    puts stderr "stooop version [package provide stooop] requires Tcl 8.3 or above"
+    exit 1
+}
 
-package provide stooop 3.3
 
 catch {rename proc _proc}            ;# rename proc before it is overloaded, ignore error in case of multiple inclusion of this file
 
@@ -14,7 +20,7 @@ namespace eval ::stooop {
 
     set checkCode {}                                ;# no checking by default: use an empty instruction to avoid any performance hit
     if {[info exists ::env(STOOOPCHECKALL)]} {
-        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {}}
+        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {} STOOOPCHECKOBJECTS {}}
     }
     if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
         append checkCode {::stooop::checkProcedure;}
@@ -57,15 +63,15 @@ namespace eval ::stooop {
         # use local variable for identifier because new can be invoked recursively
         if {[scan $classOrId %u dummy]==0} {                                                            ;# first argument is a class
             set constructor ${classOrId}::[namespace tail $classOrId]                                   ;# generate constructor name
-            # generate fully qualified class namespace name
-            set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
             # we could detect here whether class was ever declared but that would prevent stooop packages to load properly, because
             # constructor would not be invoked and thus class source file never sourced
-            set fullClass([set id [incr newId]]) $fullName
             # invoke the constructor for the class with optional arguments in caller's variable context so that object creation is
             # transparent and that array names as constructor parameters work with a simple upvar
-            uplevel $constructor $id $args
-        } else {  ;# first argument is an object identifier (unsigned integer) , copy source object to new object of identical class
+            # note: if class is in a package, the class namespace code is loaded here, as the first object of the class is created
+            uplevel $constructor [set id [incr newId]] $args
+            # generate fully qualified class namespace name now that we are sure that class namespace code has been invoked
+            set fullClass($id) [namespace qualifiers [uplevel namespace which -command $constructor]]
+        } else {   ;# first argument is an object identifier (unsigned integer), copy source object to new object of identical class
             if {[catch {set fullClass([set id [incr newId]]) $fullClass($classOrId)}]} {
                 error "invalid object identifier $classOrId"
             }
@@ -91,9 +97,7 @@ namespace eval ::stooop {
         # invoke the destructor for the class in caller's variable context so that object deletion is transparent
         uplevel ${fullClass}::~[namespace tail $fullClass] $id
         # delete all this object data members if any (assume that they were stored as ${class}::($id,memberName))
-        foreach name [array names ${fullClass}:: $id,*] {
-            unset ${fullClass}::($name)
-        }
+        array unset ${fullClass}:: $id,*
         # data member arrays deletion is left to the user
     }
 
@@ -105,8 +109,8 @@ namespace eval ::stooop {
 
     _proc copy {fullClass from to} {                                          ;# copy object data members from one object to another
         set index [string length $from]
-        foreach name [array names ${fullClass}:: $from,*] {                                             ;# copy regular data members
-            set ${fullClass}::($to[string range $name $index end]) [set ${fullClass}::($name)]
+        foreach {name value} [array get ${fullClass}:: $from,*] {                                       ;# copy regular data members
+            set ${fullClass}::($to[string range $name $index end]) $value
         }
         # if any, array data members copy is left to the class programmer through the then mandatory copy constructor
     }
@@ -135,7 +139,7 @@ _proc ::stooop::parseProcedureName {namespace name fullClassVariable procedureVa
         return 1
     } else {                                                 ;# procedure is either a member of a known class or a regular procedure
         if {![string match ::* $name]} {                                                  ;# eventually fully qualify procedure name
-            if {[string compare $namespace ::]==0} {                                                ;# global namespace special case
+            if {[string equal $namespace ::]} {                                                     ;# global namespace special case
                 set name ::$name
             } else {
                 set name ${namespace}::$name
@@ -161,20 +165,20 @@ _proc ::stooop::parseProcedureName {namespace name fullClassVariable procedureVa
 _proc ::stooop::virtual {keyword name arguments args} {
     variable pureVirtual     ;# set a flag so that proc knows it is acting upon a virtual procedure, also serves as a pure indicator
 
-    if {[string compare [uplevel namespace which -command $keyword] ::proc]!=0} {
+    if {![string equal [uplevel namespace which -command $keyword] ::proc]} {
         error "virtual operator works only on proc, not $keyword"
     }
     if {![parseProcedureName [uplevel namespace current] $name fullClass procedure message]} {
         error $message                                                                       ;# not in a member procedure definition
     }
     set class [namespace tail $fullClass]
-    if {[string compare $class $procedure]==0} {
+    if {[string equal $class $procedure]} {
         error "cannot make class $fullClass constructor virtual"
     }
-    if {[string compare ~$class $procedure]==0} {
+    if {[string equal ~$class $procedure]} {
         error "cannot make class $fullClass destructor virtual"
     }
-    if {[string compare [lindex $arguments 0] this]!=0} {
+    if {![string equal [lindex $arguments 0] this]} {
         error "cannot make static procedure $procedure of class $fullClass virtual"
     }
     set pureVirtual [expr {[llength $args]==0}]                                              ;# no procedure body means pure virtual
@@ -194,11 +198,11 @@ _proc proc {name arguments args} {
         error "missing body for ${fullClass}::$procedure"
     }
     set class [namespace tail $fullClass]
-    if {[string compare $class $procedure]==0} {                                                     ;# class constructor definition
-        if {[string compare [lindex $arguments 0] this]!=0} {
+    if {[string equal $class $procedure]} {                                                          ;# class constructor definition
+        if {![string equal [lindex $arguments 0] this]} {
             error "class $fullClass constructor first argument must be this"
         }
-        if {[string compare [lindex $arguments 1] copy]==0} {                            ;# user defined copy constructor definition
+        if {[string equal [lindex $arguments 1] copy]} {                                 ;# user defined copy constructor definition
             if {[llength $arguments]!=2} {
                 error "class $fullClass copy constructor must have 2 arguments exactly"
             }
@@ -210,11 +214,11 @@ _proc proc {name arguments args} {
             eval ::stooop::constructorDeclaration $fullClass $class 0 \{$arguments\} $args
             ::stooop::generateDefaultCopyConstructor $fullClass                          ;# always generate default copy constructor
         }
-    } elseif {[string compare ~$class $procedure]==0} {                                              ;# class destructor declaration
+    } elseif {[string equal ~$class $procedure]} {                                                   ;# class destructor declaration
         if {[llength $arguments]!=1} {
             error "class $fullClass destructor must have 1 argument exactly"
         }
-        if {[string compare [lindex $arguments 0] this]!=0} {
+        if {![string equal [lindex $arguments 0] this]} {
             error "class $fullClass destructor argument must be this"
         }
         if {[catch {info body ::${fullClass}::$class}]} {                      ;# use fastest method for testing procedure existence
@@ -238,7 +242,7 @@ _proc ::stooop::constructorDeclaration {fullClass class copy arguments args} { ;
     if {($number%2)==0} {                                                    ;# check that each base class constructor has arguments
         error "bad class $fullClass constructor declaration, a base class, contructor arguments or body may be missing"
     }
-    if {[string compare [lindex $arguments end] args]==0} {
+    if {[string equal [lindex $arguments end] args]} {
         set variable($fullClass) {}                    ;# remember that there is a variable number of arguments in class constructor
     }
     if {!$copy} {
@@ -269,8 +273,8 @@ _proc ::stooop::constructorDeclaration {fullClass class copy arguments args} { ;
             }
             lappend fullBases($fullClass) $fullBase
         }
-        # remove new lines in base arguments part in case user has formatted long declarations with new lines
-        regsub -all \n $baseArguments {} constructorArguments($fullBase)
+        # replace new lines with blanks in base arguments part in case user has formatted long declarations with new lines
+        regsub -all {\n} $baseArguments { } constructorArguments($fullBase)
     }
     # setup access to class data (an empty named array)
     # fully qualify tcl variable command for it may have been redefined within the class namespace
@@ -325,7 +329,7 @@ $checkCode
     }                                                                                     ;# else no base class derivation specified
     if {$copy} {                                        ;# for user defined copy constructor, copy derived class member if it exists
         append constructorBody \
-"::catch {::set ${fullClass}::(\$this,_derived) \[::set ${fullClass}::(\$[::lindex $arguments 1],_derived)\]}
+"::catch {::set (\$this,_derived) \$(\$[::lindex $arguments 1],_derived)}
 "
     }
     append constructorBody [lindex $args end]                                          ;# finally append user defined procedure body
@@ -368,7 +372,7 @@ _proc ::stooop::memberProcedureDeclaration {fullClass class procedure arguments
             _proc ${fullClass}::$procedure $arguments \
 "::variable {}
 $checkCode
-::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel \$(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         } else {                                                                                      ;# regular virtual declaration
             # setup access to class data
@@ -382,10 +386,10 @@ $body
             _proc ${fullClass}::$procedure $arguments \
 "::variable {}
 $checkCode
-if {!\[::catch {::info body \$${fullClass}::(\$this,_derived)::$procedure}\]} {
-::return \[::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
+if {!\[::catch {::info body \$(\$this,_derived)::$procedure}\]} {
+::return \[::uplevel \$(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
 }
-::eval ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         }
     } else {                                                                                              ;# non virtual declaration
@@ -442,23 +446,55 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
                 set interface($fullClass) {}
             }
         }
+    }
 
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        _proc invokingProcedure {} {
+            if {[catch {set procedure [lindex [info level -2] 0]}]} {                                       ;# no invoking procedure
+                return {main script}
+            } elseif {[string length $procedure]==0} {                                              ;# invoked from a namespace body
+                return "namespace [uplevel 2 namespace current]"
+            } else {                                ;# store fully qualified name, visible from creator procedure invoking procedure
+                return [uplevel 3 namespace which -command $procedure]
+            }
+        }
+    }
+
+    if {[info exists ::env(STOOOPCHECKPROCEDURES)]||[info exists ::env(STOOOPCHECKOBJECTS)]} {
         catch {rename ::stooop::new ::stooop::_new}                              ;# gracefully handle multiple sourcing of this file
         _proc ::stooop::new {classOrId args} {           ;# use a new new procedure instead of adding debugging code to existing one
-            variable fullClass
-            variable interface
-
-            if {[scan $classOrId %u dummy]==0} {                                                        ;# first argument is a class
-                set constructor ${classOrId}::[namespace tail $classOrId]                               ;# generate constructor name
-                set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-            } else {                                                                       ;# first argument is an object identifier
-                set fullName $fullClass($classOrId)
+            variable newId
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                variable fullClass
+                variable interface
             }
-            if {[info exists interface($fullName)]} {
-                error "class $fullName with pure virtual procedures should not be instanciated"
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                variable creator
+            }
+
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                if {[scan $classOrId %u dummy]==0} {                                                    ;# first argument is a class
+                    set constructor ${classOrId}::[namespace tail $classOrId]                           ;# generate constructor name
+                    catch {$constructor}          ;# force loading in case class is in a package so namespace commands work properly
+                    set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
+                    # anticipate full class name storage in original new{} in order to avoid invalid object identifier error in
+                    # checkProcedure{} when member procedure is invoked from within contructor, in which case full class name would
+                    # have yet to be stored.
+                    set fullClass([expr {$newId+1}]) $fullName             ;# new identifier is really incremented in original new{}
+                } else {                                                                   ;# first argument is an object identifier
+                    set fullName $fullClass($classOrId)                     ;# class code, if from a package, must already be loaded
+                }
+                if {[info exists interface($fullName)]} {
+                    error "class $fullName with pure virtual procedures should not be instanciated"
+                }
+            }
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                # keep track of procedure in which creation occured (new identifier is really incremented in original new{})
+                set creator([expr {$newId+1}]) [invokingProcedure]
             }
             return [uplevel ::stooop::_new $classOrId $args]
         }
+
     }
 
     _proc ::stooop::ancestors {fullClass} {                              ;# return the unsorted list of ancestors in class hierarchy
@@ -492,12 +528,12 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
         set procedure [lindex $list 0]
         set fullProcedure [uplevel 3 namespace which -command $procedure]            ;# procedure must be known at the invoker level
         set procedure [namespace tail $procedure]                                                            ;# strip procedure name
-        if {[string compare $class $procedure]==0} {                                                                  ;# constructor
+        if {[string equal $class $procedure]} {                                                                       ;# constructor
             set procedure constructor
-        } elseif {[string compare ~$class $procedure]==0} {                                                            ;# destructor
+        } elseif {[string equal ~$class $procedure]} {                                                                 ;# destructor
             set procedure destructor
         }
-        if {[string compare [lindex [info args $fullProcedure] 0] this]==0} {                                ;# non static procedure
+        if {[string equal [lindex [info args $fullProcedure] 0] this]} {                                     ;# non static procedure
             set thisParameter [lindex $list 1]                                                ;# object identifier is first argument
         }
     }
@@ -507,11 +543,12 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
 
         debugInformation class qualifiedClass procedure qualifiedProcedure this
         if {![info exists this]} return                                                    ;# static procedure, no checking possible
+        if {[string equal $procedure constructor]} return        ;# in constructor, checking useless since object is not yet created
         if {![info exists fullClass($this)]} {
             error "$this is not a valid object identifier"
         }
         set fullName [string trimleft $fullClass($this) :]
-        if {[string compare $fullName $qualifiedClass]==0} return                              ;# procedure and object classes match
+        if {[string equal $fullName $qualifiedClass]} return                                   ;# procedure and object classes match
         # restore global qualifiers to compare with internal full class array data
         if {[lsearch -exact [ancestors ::$fullName] ::$qualifiedClass]<0} {
             error "class $qualifiedClass of $qualifiedProcedure procedure not an ancestor of object $this class $fullName"
@@ -543,20 +580,20 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
     # then if procedure is not static, check that only data belonging to the object passed as parameter is accessed
     _proc ::stooop::checkData {array name operation} {
         scan $name %u,%s identifier member
-        if {[info exists member]&&([string compare $member _derived]==0)} return                ;# ignore internally defined members
+        if {[info exists member]&&[string equal $member _derived]} return                       ;# ignore internally defined members
 
         debugInformation class qualifiedClass procedure qualifiedProcedure this
         if {![info exists class]} return                                     ;# no checking can be done outside of a class namespace
         set array [uplevel [list namespace which -variable $array]]                                     ;# determine array full name
         if {![info exists procedure]} {                                                                  ;# inside a class namespace
-            if {[string compare $array ::${qualifiedClass}::]!=0} {           ;# compare with empty named array fully qualified name
+            if {![string equal $array ::${qualifiedClass}::]} {               ;# compare with empty named array fully qualified name
                 # trace command error message is automatically prepended and indicates operation
                 error "class access violation in class $qualifiedClass namespace"
             }
             return                                                                                                           ;# done
         }
-        if {[string compare $qualifiedProcedure ::stooop::copy]==0} return                         ;# ignore internal copy procedure
-        if {[string compare $array ::${qualifiedClass}::]!=0} {               ;# compare with empty named array fully qualified name
+        if {[string equal $qualifiedProcedure ::stooop::copy]} return                              ;# ignore internal copy procedure
+        if {![string equal $array ::${qualifiedClass}::]} {                   ;# compare with empty named array fully qualified name
             # trace command error message is automatically prepended and indicates operation
             error "class access violation in procedure $qualifiedProcedure"
         }
@@ -573,10 +610,10 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
         variable traceDataFormat
 
         scan $name %u,%s identifier member
-        if {[info exists member]&&([string compare $member _derived]==0)} return                ;# ignore internally defined members
+        if {[info exists member]&&[string equal $member _derived]} return                       ;# ignore internally defined members
 
         # ignore internal destruction
-        if {![catch {lindex [info level -1] 0} procedure]&&([string compare ::stooop::deleteObject $procedure]==0)} return
+        if {![catch {lindex [info level -1] 0} procedure]&&[string equal ::stooop::deleteObject $procedure]} return
         set class {}                                                                               ;# in case we are outside a class
         set qualifiedClass {}
         set procedure {}                                                                 ;# in case we are outside a class procedure
@@ -602,11 +639,56 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
         }
         array set string {r read w write u unset}
         regsub -all %o $text $string($operation) text
-        if {[string compare $operation u]==0} {
+        if {[string equal $operation u]} {
             regsub -all %v $text {} text                                                                  ;# no value when unsetting
         } else {
             regsub -all %v $text [uplevel set ${array}($name)] text
         }
         puts $traceDataChannel $text
     }
+
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        # print existing objects along with creation procedure, with optional class pattern (see the string Tcl command manual)
+        _proc ::stooop::printObjects {{pattern *}} {
+            variable fullClass
+            variable creator
+
+            puts "stooop::printObjects invoked from [invokingProcedure]:"
+            foreach id [lsort -integer [array names fullClass]] {
+                if {[string match $pattern $fullClass($id)]} {
+                    puts "$fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+
+        _proc ::stooop::record {} {                                                  ;# record all existing objects for later report
+            variable fullClass
+            variable checkpointFullClass
+
+            puts "stooop::record invoked from [invokingProcedure]"
+            catch {unset checkpointFullClass}
+            array set checkpointFullClass [array get fullClass]
+        }
+
+        _proc ::stooop::report {{pattern *}} {     ;# print all new or deleted object since last record, with optional class pattern
+            variable fullClass
+            variable checkpointFullClass
+            variable creator
+
+            puts "stooop::report invoked from [invokingProcedure]:"
+            set checkpointIds [lsort -integer [array names checkpointFullClass]]
+            set currentIds [lsort -integer [array names fullClass]]
+            foreach id $currentIds {
+                if {[string match $pattern $fullClass($id)]&&([lsearch -exact $checkpointIds $id]<0)} {
+                    puts "+ $fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+            foreach id $checkpointIds {
+                if {[string match $pattern $checkpointFullClass($id)]&&([lsearch -exact $currentIds $id]<0)} {
+                    puts "- $checkpointFullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+    }
+
 }
